"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = MetaUpdater;
var _react = _interopRequireDefault(require("react"));
var _reactRedux = require("react-redux");
var _isEqual = _interopRequireDefault(require("lodash/isEqual.js"));
var _mergeMeta = _interopRequireDefault(require("../../meta/mergeMeta.js"));
var _applyMeta = _interopRequireDefault(require("../../meta/applyMeta.js"));
var _dropUndefinedProperties = _interopRequireDefault(require("../../meta/dropUndefinedProperties.js"));
var _context = require("../../context.js");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function MetaUpdater(_ref) {
  var meta = _ref.meta,
    stash = _ref.stash,
    routePosition = _ref.routePosition,
    props = _ref.props,
    customProps = _ref.customProps;
  // `meta()` function uses `useSelector()` hook,

  // Calculate `newMeta` even if `InitialMetaHasBeenApplied` flag is `false`,
  // so that it calls the `useSelector()` hook which will trigger a rerender later
  // when the value returned by that `useSelector()` hook changes.
  var newMeta = meta({
    useSelector: function useSelector(getter) {
      return (0, _reactRedux.useSelector)(getter);
    },
    props: _objectSpread(_objectSpread({}, props), customProps)
  });
  if (newMeta) {
    newMeta = (0, _dropUndefinedProperties["default"])(newMeta);
  }
  if ((0, _context.getFromContext)('App/InitialMetaHasBeenApplied')) {
    var prevMeta;
    switch (routePosition) {
      case 'root':
        prevMeta = stash.getRootRouteComponentMeta();
        break;
      case 'leaf':
        prevMeta = stash.getPageRouteComponentMeta();
        break;
      default:
        throw new Error("[react-pages] Unsupported route position: \"".concat(routePosition, "\""));
    }
    if (!(0, _isEqual["default"])(prevMeta, newMeta)) {
      switch (routePosition) {
        case 'root':
          (0, _applyMeta["default"])((0, _mergeMeta["default"])({
            rootMetaData: newMeta,
            pageMetaData: stash.getPageRouteComponentMeta(),
            stash: stash
          }));
          console.log('[react-pages] Root <meta/> updated');
          break;
        case 'leaf':
          (0, _applyMeta["default"])((0, _mergeMeta["default"])({
            rootMetaData: stash.getRootRouteComponentMeta(),
            pageMetaData: newMeta,
            stash: stash
          }));
          console.log('[react-pages] Page <meta/> updated');
          break;
        default:
          throw new Error("[react-pages] Unsupported route position: \"".concat(routePosition, "\""));
      }
    }
  }
  return null;
}

// MetaUpdater.propTypes = {
// 	meta: PropTypes.func.isRequired,
// 	stash: PropTypes.object.isRequired,
// 	routePosition: PropTypes.oneOf(['root', 'leaf']).isRequired,
// 	props: PropTypes.object,
// 	customProps: PropTypes.object
// }
//# sourceMappingURL=MetaUpdater.js.map