"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createRouterElement;
var _react = _interopRequireDefault(require("react"));
var _found = require("@catamphetamine/found");
var _ScrollManager = _interopRequireDefault(require("./found-scroll/ScrollManager.js"));
var _RouteProvider = _interopRequireDefault(require("../RouteProvider.js"));
var _render2 = _interopRequireDefault(require("../render.js"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
// `@catamphetamine/found` is a fork of `found` with some changes:
// * `redux` and `react-redux` are `peerDependencies` instead of `dependencies`.
// * `farce` was replaced with `@catamphetamine/farce`.
// * Fixed a bug when `found` ignores all navigation actions until its `componentDidMount()` is called.

// It turned out that including a modified version of just `ScrollManager.js`
// is simpler that maintaining a fork of `found-scroll`.
// // `@catamphetamine/found-scroll` is a fork of `found-scroll` with some changes
// // * `found` in `peerDependencies` instead of dependencies.
// // * `found` replaced with `@catamphetamine/found`.
// import { ScrollManager } from '@catamphetamine/found-scroll'

// Rewrote `<ScrollManager/>` from `found-scroll` in React hooks.
// Also fixed React strict mode bug.
// https://github.com/4Catalyzer/found-scroll/issues/382

function createRouterElement(renderArgs, _ref) {
  var dispatch = _ref.dispatch,
    getState = _ref.getState;
  var ConnectedRouter = (0, _found.createConnectedRouter)({
    render: function render(renderArgs) {
      // Force re-mount the last route component on location path change.
      // https://github.com/4Catalyzer/found/issues/199#issuecomment-415616836
      var elements = renderArgs.elements;
      if (elements && window.reactPagesRemountOnNavigate !== false) {
        elements[elements.length - 1] = _react["default"].cloneElement(elements[elements.length - 1], {
          key: renderArgs.location.pathname
        });
      }
      // When a user navigates to a page, this `render()` function is called:
      // * If there's any `load()` data loader:
      //   * The first call is gonna be with `elements: undefined` and the new `location`,
      //     before the page starts loading the initial data.
      //     `<RouteProvider/>` ignores this `render()` call.
      //   * The second call is gonna be with `elements: React.Element[]` and the new `location`,
      //     after the page has loaded the initial data.
      // * Otherwise, if there's no data to load:
      //   * The page just renders with `elements: React.Element[]` and the new `location`.
      return _react["default"].createElement(_RouteProvider["default"], {
        // `elements` is `undefined` when router starts loading the next page.
        // In that case, the previous page is still rendered, so it shouldn't
        // pass the new `location` to `<RouteProvider/>` yet.
        location: elements && renderArgs.location,
        params: elements && renderArgs.params,
        routes: elements && renderArgs.routes
      }, _react["default"].createElement(_ScrollManager["default"], {
        renderArgs: renderArgs
      }, (0, _render2["default"])(renderArgs)));
    }
  });
  return _react["default"].createElement(ConnectedRouter, {
    matchContext: {
      dispatch: dispatch,
      getState: getState
    },
    resolver: _found.resolver,
    initialRenderArgs: renderArgs
  });
}
//# sourceMappingURL=createRouterElement.js.map